######################################################################
##### Code for Day 4 of SEM Workshop
##### covering Composite Variables & Advanced Topics
#####
##### Jarrett E.K. Byrnes
#####
##### Last Modified 2/1/13
######################################################################

#Set your working Directory
#setwd("~/Dropbox/SEM/lavaan")


##############################################################################################################################
# Composite Variables for Nonlinearities
##############################################################################################################################


#Load your Data File
keeley<-read.csv("./Keeley_rawdata_select4.csv")


#is this a nonlinear relationship?
plot(rich ~ cover, data=keeley, cex.lab=1.5, cex.axis=1.3)

#compare models
linear<-lm(rich ~ cover, data=keeley)
nonlinear<-lm(rich ~ cover+I(cover^2), data=keeley)

library(AICcmodavg)
aictab(list(linear, nonlinear), c("linear", "squared"))

#plot the nonlinear result
curve(coef(nonlinear)[1] + coef(nonlinear)[2]*x + coef(nonlinear)[3]*x^2, add=T, col="red", lwd=4)
curve(coef(linear)[1] + coef(linear)[2]*x, add=T, col="blue", lwd=4)

#Step 1: Create a new nonlinear variable in the data
keeley<-within(keeley, coverSQ<-cover^2)

#Now, we can fit this with no composite
noCompModel <- 'rich ~ cover + coverSQ'

noCompFit <- sem(noCompModel, data=keeley)
summary(noCompFit, rsquare=T)

#Now, add a composite

compModel<-'

					coverEffect <~ 58*cover + coverSQ

				  rich ~ coverEffect'

compFit <- sem(compModel, data=keeley)

summary(compFit)

###
#Exercise: Abiotic
###

#how about abotic and fire severity?
plot(firesev ~ abiotic, data=keeley)

fireLinear<-lm(firesev ~ abiotic, data=keeley)
fireNonlinear<-lm(firesev ~ abiotic + I(abiotic^2), data=keeley)

aictab(list(fireLinear, fireNonlinear), c("linear", "squared"))

curve(coef(fireNonlinear)[1] + coef(fireNonlinear)[2]*x + coef(fireNonlinear)[3]*x^2, add=T, col="red", lwd=4)

keeley$abioticSQ <- keeley$abiotic^2



abioticCompModelBad<-'
                   abioticEffect <~ 0.4 * abiotic + abioticSQ

                  firesev ~ abioticEffect'

abioticCompModel<-'
                   abioticEffect <~ abiotic + abioticSQ

                  firesev ~ 1*abioticEffect'

abioticCompFit <- sem(abioticCompModel, data=keeley)

summary(abioticCompFit)

####
# Endogenous composite
####


endoCompModel<-'
                coverEffect <~ 1*cover + coverSQ

                cover ~~ coverSQ
                age ~~ coverSQ

                cover ~ age
                rich ~ age + coverEffect'


endoCompFit <- sem(endoCompModel, data=keeley, fixed.x=F)

summary(endoCompFit)

##############################################################################################################################
# Multiple Categorical Predictors
##############################################################################################################################

urchinData<-read.csv("./urchin_ex_sem.csv")


#Make treatment into a series of binary variables
source("./makeBinaryTreatments.R")
binTrt<-makeBinaryTreatments(urchinData, "treatment")

#add new columns to data frame
urchinData<-cbind(urchinData, binTrt)

#The observed variable model

urchinModel<-' 
  Feeding.rate.dry ~ MAPY + R
  GONAD_INDEX ~ MAPY + R + Feeding.rate.dry
'

urchinSEM<-sem(urchinModel, data=urchinData)

summary(urchinSEM, rsquare=T, standardized=T)

#now make a composite model


urchinCompositeModel<-' 
    Treatment <~ MAPY + .002*R
  
    Feeding.rate.dry ~ Treatment

    GONAD_INDEX ~ Treatment + Feeding.rate.dry
'

urchinCompositeSEM<-sem(urchinCompositeModel, data=urchinData)

summary(urchinCompositeSEM, rsquare=T, standardized=T)

anova(urchinSEM, urchinCompositeSEM)

########################
####Exercise
########################

urchinModelTest<-' 
  Feeding.rate.dry ~ MAPY + R
  TEST_CHANGE ~ MAPY + R + Feeding.rate.dry
'

urchinSEMT<-sem(urchinModelTest, data=urchinData)
inspect(urchinSEMT, "sample")


#So, fix test change!
urchinData$TEST_CHANGE_10<-urchinData$TEST_CHANGE/10



urchinModel2<-' 
  Feeding.rate.dry ~ MAPY + R
  TEST_CHANGE_10 ~ MAPY + R + Feeding.rate.dry
'

urchinSEM2<-sem(urchinModel2, data=urchinData)
summary(urchinSEM2)


urchinCompositeModel2<-' 
  Treatment <~ MAPY + .002*R

  Feeding.rate.dry ~ Treatment
  TEST_CHANGE_10 ~Treatment + Feeding.rate.dry
'
urchinCompositeFit2<-sem(urchinCompositeModel2, data=urchinData)

summary(urchinCompositeFit2)


##############################################################################################################################
# Advanced Topics
##############################################################################################################################

#######
## total and indirect effects
#######



#Load your Data File
keeley<-read.csv("./Keeley_rawdata_select4.csv")


partialMedModel<-' firesev ~ af*age
                cover ~ fc*firesev + ac*age
          
                indirect := af*fc
                total := ac + (af*fc)

'

partialMedSEM<-sem(partialMedModel, 
                   data=keeley)

#look at coefficients
summary(partialMedSEM)

##############################################################################################################################
# D-Sep Revisited
##############################################################################################################################


#########
# D-Sep in lavaan
#########
#work on this...
#Full Mediation
distModel2 <- 'rich ~ abiotic  + hetero
              hetero ~ distance
              abiotic ~ distance'

distFit2 <- sem(distModel2, data=keeley)

source("./dsepTest.R")

dsepTest(distFit2)
dsepTest(distFit2, showall=T)

#########
# D-Sep from Shipley 2009
#The Example from the appendix
#########

Shipley<-read.table("./Shipley.dat")

pairs(Shipley)


library(ggplot2)
qplot(DD, Date, data=Shipley, facets=~site, colour=tree) + theme_bw(base_size=16) 

####How would we fit paths?
#e.g., for DD -> lat
library(nlme)

#model with random intercept
#tree nested in site
Date_dd<-lme(Date~DD,data=Shipley,
             random=~1|site/tree,na.action=na.omit)



#Independence claim: (Date,lat)|{DD} 
fit1<-lme(Date~DD+lat,data=Shipley,random=~1|site/tree,na.action=na.omit)
summary(fit1)$tTable #9.373049e-01

#Independence claim: (Growth,lat)|{Date} 
fit2<-lme(Growth~Date+lat,data=Shipley,random=~1|site/tree,na.action=na.omit) 
summary(fit2)$tTable #3.836896e-01


#Independence claim: (Growth,DD)|{Date,lat} 
fit3<-lme(Growth~Date+lat+DD,data=Shipley,random=~1|site/tree,na.action=na.omit)
summary(fit3)$tTable #7.667083e-01

###need lme4 for the glmms
library(lme4)
qplot(Growth, Survival, data=Shipley, facets=~site, colour=tree) + theme_bw(base_size=16) 


#Independence claim with glmm (Live,lat)|{Growth}
fit4<-lmer(Live~Growth+lat+(1|site)+(1|tree),data=Shipley, na.action=na.omit, family=binomial(link="logit"))
summary(fit4)@coefs #2.791099e-01

#Independence claim with glmm (Live,DD)|{Growth,lat} 
fit5<-lmer(Live~Growth+lat+DD+(1|site)+(1|tree),data=Shipley,na.action=na.omit, family=binomial(link="logit"))
summary(fit5)@coefs #3.159286e-01

#Independence claim with glmm (Live,Date)|{DD,Growth} 
fit6<-lmer(Live~Growth +DD+Date+(1|site)+(1|tree),data=Shipley,na.action=na.omit, family=binomial(link="logit"))
summary(fit6)@coefs #1.519170e-01


#note, since we're logging things
#we can use log(a)+log(b) = log(a*b)
fisherC <- -2* log(9.373049e-01 * 3.836896e-01 * 7.667083e-01 * 2.791099e-01 * 3.159286e-01 * 1.519170e-01)

1-pchisq(fisherC, 2*6)

#AIC
#each model has 5 parameters - slope, intercept, 
#variance, and random variance for slope & intercept, so, K=5*4
fisherC + 2*(5*4) 


########################################################
####### Latent Growth Curve Models
########################################################

kfmTseries<-read.csv("./kfmTseries.csv")

gMod<-'
  Initial =~ 1*KelpT1 + 1*KelpT2 + 1*KelpT3 + 1*KelpT4
	Growth =~ 0*KelpT1 + 1*KelpT2 + 2*KelpT3 + 3*KelpT4
	KelpT2 ~~ KelpT1
	KelpT3 ~~ KelpT2
	KelpT4 ~~ KelpT3
	'
gFit<-growth(gMod, data=kfmTseries)

summary(gFit, rsquare=T)

gMod2<-'
  Initial =~ 1*KelpT1 + 1*KelpT2 + 1*KelpT3 + 1*KelpT4
  Growth =~ 0*KelpT1 + 1*KelpT2 + 2*KelpT3 + 3*KelpT4
	KelpT2 ~ a*KelpT1
	KelpT3 ~ a*KelpT2
	KelpT4 ~ a*KelpT3
	'
gFit2<-growth(gMod2, data=kfmTseries)

#compare
anova(gFit, gFit2)

summary(gFit2, rsquare=T)

nutrientGMod <-paste(gMod2, '
	
	Initial ~ Mean.Benthic.Temp
	Growth ~ Mean.Benthic.Temp

	', sep="\n")

nutrientGFit<-growth(nutrientGMod, data=kfmTseries)

summary(nutrientGFit)