###
##Implements Shipley's D-Separation method for SEMs with no latent variables, correlated errorrs, or cycles
##
## JEB
##
## Last Modified 8/4/2011
## Changelog
## 8/4/11 - now independent of ggm!  Also, added ability to handle multiple groups

dsepTest<-function(object, showall=F, adjust="none"){
	  
  cmats<-inspect(object, "coef")
  if(length(lavaanNames(object, type="lv") > 0)) stop("This SEM contains latent variables!")
  if(sum(cmats$theta[lower.tri(cmats$theta)]) != 0) stop("This SEM contains correlated errors.  Method not yet implemented, but see Shipley 2003.")

  dag<-DGlavaan(object)

  #check for being recursive
  if (!isAcyclic(dag)) stop("The SEM contains directed cycles!")
  
  #calculate Shipley's C (of Fisher's C)
  covMat<-object@SampleStats@cov
  n<-object@Data@nobs
  
  #there may be one or more groups, so, generalized method to deal with this
  rlist<-lapply(1:object@Data@ngroups, function(i) {

  	dimnames(covMat[[i]])<-dimnames(dag)
  	n<-object@Data@nobs[[i]]
  	return(shipleyC(dag, covMat[[i]], n, showall=showall, adjust=adjust) )
  })

  if(object@Data@ngroups==1) return(rlist[[1]]) #if only 1 group, just return the shipleyC list

  #otherwise, add group names
  names(rlist)<-object@Data@group.label
  
  return(rlist)

}


####
# gets the directed graph from a lavaan object
DGlavaan<-function(object){
  cmats<-inspect(object, "coef")
    
  dg<-t(cmats$beta)
  dg<-(dg !=0)+0 #translate to 1s and 0s
  
  return(dg)
}

####
# determines if a graph is acyclic or not
####
isAcyclic<-function(amat){
	cycles<- (amat-t(amat)==1) + (amat==0)
	return(length(which(cycles==0)) == 0)
	}

###
#based on shipley.test from ggm library 1.0.4, but, this one also lets one see the dsep table itself
#if showall=T
#hrm...maybe add a pval for a normal distribution?
####

shipleyC<-function (amat, S, n, showall=F, adjust="none") 
{
    pval <- function(r, q, n) {
        df = n - 2 - q
        tval <- r * sqrt(df)/sqrt(1 - r * r)
        2 * pt(-abs(tval), df)
    }
    l <- basiSet(amat)
    k <- length(l)
    p <- rep(0, k)
    for (i in 1:k) {
        r <- pcor(l[[i]], S)
        q <- length(l[[i]]) - 2
        p[i] <- pval(r, q, n)
    }
    ctest <- -2 * sum(log(p))
    df <- 2 * k
    pv <- 1 - pchisq(ctest, df)
    ret<-list(ctest = ctest, df = df, pvalue = pv)

    if(!showall) return(ret)
    
    #to see tests for individual elements of the basis set - this is the new material - JEB
    ret$dsep<-sapply(l, function(x) {
    	vars<-c(paste(x[1], x[2], sep=","), "")
    	if(length(x)>2) vars[2] <- paste(x[3:length(x)], collapse=",")
    	return(vars)
    	})

   ret$dsep<-data.frame(Pair=ret$dsep[1,], Conditioning=ret$dsep[2,], "P(t)"=p.adjust(p, adjust))
    ret
}

####
##function to calculate partial correlations
##taken from ggm 1.0.4 to avoid having a dependency
####

pcor<-function (u, S) 
{
    k <- solve(S[u, u])
    -k[1, 2]/sqrt(k[1, 1] * k[2, 2])
}

####
##gets the basis set from a DAG
####

basiSet<-function(amat){
  if (!isAcyclic(amat)) stop("This graph contains cycles!")
	
  set<-which(amat+t(amat)==0 & lower.tri(amat), arr.ind=T) #get the vertices

  #get their parents and smoosh them together along with the vertices...and look up their names! 
  apply(set, 1, function(arow) {
  	colnames(amat)[c(arow, unique(c(which(amat[,arow[1]]==1), which(amat[,arow[2]]==1))))]
  })	
}


# # set.seed(2001)

# adf<-rbind(data.frame(x=rnorm(100, sd=20)), data.frame(x=rnorm(100, sd=20)))

# adf<-within(adf,{
	# grp = c(rep("A", 100), rep("B", 100))
	# y1<-rnorm(100, 20*x, sd=30)
	# y2<-rnorm(100, 10*x, sd=40)
	# y3 <- rnorm(100, 40*x+20*y2, sd=40)
	# })

# mod<-"y1 ~ x\ny2~x\ny3 ~ x+y2"

# asem<-sem(mod, data=adf)
# dsepTest(asem, showall=T)


# asem<-sem(mod, data=adf, group="grp")
# dsepTest(asem, showall=T)


